# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import loki
from ..models import dashboard


class Dataquery(cogbuilder.Builder[loki.Dataquery]):    
    _internal: loki.Dataquery

    def __init__(self):
        self._internal = loki.Dataquery()

    def build(self) -> loki.Dataquery:
        return self._internal    
    
    def expr(self, expr: str) -> typing.Self:    
        """
        The LogQL query.
        """
            
        self._internal.expr = expr
    
        return self
    
    def legend_format(self, legend_format: str) -> typing.Self:    
        """
        Used to override the name of the series.
        """
            
        self._internal.legend_format = legend_format
    
        return self
    
    def max_lines(self, max_lines: int) -> typing.Self:    
        """
        Used to limit the number of log rows returned.
        """
            
        self._internal.max_lines = max_lines
    
        return self
    
    def resolution(self, resolution: int) -> typing.Self:    
        """
        @deprecated, now use step.
        """
            
        self._internal.resolution = resolution
    
        return self
    
    def editor_mode(self, editor_mode: loki.QueryEditorMode) -> typing.Self:        
        self._internal.editor_mode = editor_mode
    
        return self
    
    def range_val(self, range_val: bool) -> typing.Self:    
        """
        @deprecated, now use queryType.
        """
            
        self._internal.range_val = range_val
    
        return self
    
    def instant(self, instant: bool) -> typing.Self:    
        """
        @deprecated, now use queryType.
        """
            
        self._internal.instant = instant
    
        return self
    
    def step(self, step: str) -> typing.Self:    
        """
        Used to set step value for range queries.
        """
            
        self._internal.step = step
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        If hide is set to true, Grafana will filter out the response(s) associated with this query before returning it to the panel.
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    


# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import variants as cogvariants
from ..models import dashboard
from ..cog import runtime as cogruntime


class CSVWave:
    labels: typing.Optional[str]
    name: typing.Optional[str]
    time_step: typing.Optional[int]
    values_csv: typing.Optional[str]

    def __init__(self, labels: typing.Optional[str] = None, name: typing.Optional[str] = None, time_step: typing.Optional[int] = None, values_csv: typing.Optional[str] = None):
        self.labels = labels
        self.name = name
        self.time_step = time_step
        self.values_csv = values_csv

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.labels is not None:
            payload["labels"] = self.labels
        if self.name is not None:
            payload["name"] = self.name
        if self.time_step is not None:
            payload["timeStep"] = self.time_step
        if self.values_csv is not None:
            payload["valuesCSV"] = self.values_csv
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "labels" in data:
            args["labels"] = data["labels"]
        if "name" in data:
            args["name"] = data["name"]
        if "timeStep" in data:
            args["time_step"] = data["timeStep"]
        if "valuesCSV" in data:
            args["values_csv"] = data["valuesCSV"]        

        return cls(**args)


class NodesQuery:
    count: typing.Optional[int]
    seed: typing.Optional[int]
    # Possible enum values:
    #  - `"random"` 
    #  - `"random edges"` 
    #  - `"response_medium"` 
    #  - `"response_small"` 
    #  - `"feature_showcase"` 
    type_val: typing.Optional[typing.Literal["random", "random edges", "response_medium", "response_small", "feature_showcase"]]

    def __init__(self, count: typing.Optional[int] = None, seed: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["random", "random edges", "response_medium", "response_small", "feature_showcase"]] = None):
        self.count = count
        self.seed = seed
        self.type_val = type_val

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.count is not None:
            payload["count"] = self.count
        if self.seed is not None:
            payload["seed"] = self.seed
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "count" in data:
            args["count"] = data["count"]
        if "seed" in data:
            args["seed"] = data["seed"]
        if "type" in data:
            args["type_val"] = data["type"]        

        return cls(**args)


class PulseWaveQuery:
    off_count: typing.Optional[int]
    off_value: typing.Optional[float]
    on_count: typing.Optional[int]
    on_value: typing.Optional[float]
    time_step: typing.Optional[int]

    def __init__(self, off_count: typing.Optional[int] = None, off_value: typing.Optional[float] = None, on_count: typing.Optional[int] = None, on_value: typing.Optional[float] = None, time_step: typing.Optional[int] = None):
        self.off_count = off_count
        self.off_value = off_value
        self.on_count = on_count
        self.on_value = on_value
        self.time_step = time_step

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.off_count is not None:
            payload["offCount"] = self.off_count
        if self.off_value is not None:
            payload["offValue"] = self.off_value
        if self.on_count is not None:
            payload["onCount"] = self.on_count
        if self.on_value is not None:
            payload["onValue"] = self.on_value
        if self.time_step is not None:
            payload["timeStep"] = self.time_step
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "offCount" in data:
            args["off_count"] = data["offCount"]
        if "offValue" in data:
            args["off_value"] = data["offValue"]
        if "onCount" in data:
            args["on_count"] = data["onCount"]
        if "onValue" in data:
            args["on_value"] = data["onValue"]
        if "timeStep" in data:
            args["time_step"] = data["timeStep"]        

        return cls(**args)


class ResultAssertions:
    # Maximum frame count
    max_frames: typing.Optional[int]
    # Type asserts that the frame matches a known type structure.
    # Possible enum values:
    #  - `""` 
    #  - `"timeseries-wide"` 
    #  - `"timeseries-long"` 
    #  - `"timeseries-many"` 
    #  - `"timeseries-multi"` 
    #  - `"directory-listing"` 
    #  - `"table"` 
    #  - `"numeric-wide"` 
    #  - `"numeric-multi"` 
    #  - `"numeric-long"` 
    #  - `"log-lines"` 
    type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]]
    # TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
    # contract documentation https://grafana.github.io/dataplane/contract/.
    type_version: list[int]

    def __init__(self, max_frames: typing.Optional[int] = None, type_val: typing.Optional[typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]] = None, type_version: typing.Optional[list[int]] = None):
        self.max_frames = max_frames
        self.type_val = type_val
        self.type_version = type_version if type_version is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "typeVersion": self.type_version,
        }
        if self.max_frames is not None:
            payload["maxFrames"] = self.max_frames
        if self.type_val is not None:
            payload["type"] = self.type_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "maxFrames" in data:
            args["max_frames"] = data["maxFrames"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "typeVersion" in data:
            args["type_version"] = data["typeVersion"]        

        return cls(**args)


class Key:
    tick: float
    type_val: str
    uid: typing.Optional[str]

    def __init__(self, tick: float = 0, type_val: str = "", uid: typing.Optional[str] = None):
        self.tick = tick
        self.type_val = type_val
        self.uid = uid

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "tick": self.tick,
            "type": self.type_val,
        }
        if self.uid is not None:
            payload["uid"] = self.uid
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "tick" in data:
            args["tick"] = data["tick"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "uid" in data:
            args["uid"] = data["uid"]        

        return cls(**args)


class SimulationQuery:
    config: typing.Optional[object]
    key: 'Key'
    last: typing.Optional[bool]
    stream: typing.Optional[bool]

    def __init__(self, config: typing.Optional[object] = None, key: typing.Optional['Key'] = None, last: typing.Optional[bool] = None, stream: typing.Optional[bool] = None):
        self.config = config
        self.key = key if key is not None else Key()
        self.last = last
        self.stream = stream

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "key": self.key,
        }
        if self.config is not None:
            payload["config"] = self.config
        if self.last is not None:
            payload["last"] = self.last
        if self.stream is not None:
            payload["stream"] = self.stream
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "config" in data:
            args["config"] = data["config"]
        if "key" in data:
            args["key"] = Key.from_json(data["key"])
        if "last" in data:
            args["last"] = data["last"]
        if "stream" in data:
            args["stream"] = data["stream"]        

        return cls(**args)


class StreamingQuery:
    bands: typing.Optional[int]
    noise: float
    speed: float
    spread: float
    # Possible enum values:
    #  - `"fetch"` 
    #  - `"logs"` 
    #  - `"signal"` 
    #  - `"traces"` 
    type_val: typing.Literal["fetch", "logs", "signal", "traces"]
    url: typing.Optional[str]

    def __init__(self, bands: typing.Optional[int] = None, noise: float = 0, speed: float = 0, spread: float = 0, type_val: typing.Optional[typing.Literal["fetch", "logs", "signal", "traces"]] = None, url: typing.Optional[str] = None):
        self.bands = bands
        self.noise = noise
        self.speed = speed
        self.spread = spread
        self.type_val = type_val if type_val is not None else "fetch"
        self.url = url

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "noise": self.noise,
            "speed": self.speed,
            "spread": self.spread,
            "type": self.type_val,
        }
        if self.bands is not None:
            payload["bands"] = self.bands
        if self.url is not None:
            payload["url"] = self.url
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "bands" in data:
            args["bands"] = data["bands"]
        if "noise" in data:
            args["noise"] = data["noise"]
        if "speed" in data:
            args["speed"] = data["speed"]
        if "spread" in data:
            args["spread"] = data["spread"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "url" in data:
            args["url"] = data["url"]        

        return cls(**args)


class TimeRange:
    # From is the start time of the query.
    from_val: str
    # To is the end time of the query.
    to: str

    def __init__(self, from_val: str = "now-6h", to: str = "now"):
        self.from_val = from_val
        self.to = to

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "from": self.from_val,
            "to": self.to,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "from" in data:
            args["from_val"] = data["from"]
        if "to" in data:
            args["to"] = data["to"]        

        return cls(**args)


class USAQuery:
    fields: typing.Optional[list[str]]
    mode: typing.Optional[str]
    period: typing.Optional[str]
    states: typing.Optional[list[str]]

    def __init__(self, fields: typing.Optional[list[str]] = None, mode: typing.Optional[str] = None, period: typing.Optional[str] = None, states: typing.Optional[list[str]] = None):
        self.fields = fields
        self.mode = mode
        self.period = period
        self.states = states

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.fields is not None:
            payload["fields"] = self.fields
        if self.mode is not None:
            payload["mode"] = self.mode
        if self.period is not None:
            payload["period"] = self.period
        if self.states is not None:
            payload["states"] = self.states
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "fields" in data:
            args["fields"] = data["fields"]
        if "mode" in data:
            args["mode"] = data["mode"]
        if "period" in data:
            args["period"] = data["period"]
        if "states" in data:
            args["states"] = data["states"]        

        return cls(**args)


class Dataquery(cogvariants.Dataquery):
    alias: typing.Optional[str]
    # Used for live query
    channel: typing.Optional[str]
    csv_content: typing.Optional[str]
    csv_file_name: typing.Optional[str]
    csv_wave: typing.Optional[list['CSVWave']]
    # The datasource
    datasource: typing.Optional[dashboard.DataSourceRef]
    # Drop percentage (the chance we will lose a point 0-100)
    drop_percent: typing.Optional[float]
    # Possible enum values:
    #  - `"frontend_exception"` 
    #  - `"frontend_observable"` 
    #  - `"server_panic"` 
    error_type: typing.Optional[typing.Literal["frontend_exception", "frontend_observable", "server_panic"]]
    flamegraph_diff: typing.Optional[bool]
    # true if query is disabled (ie should not be returned to the dashboard)
    # NOTE: this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Interval is the suggested duration between time points in a time series query.
    # NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
    # from the interval required to fill a pixels in the visualization
    interval_ms: typing.Optional[float]
    labels: typing.Optional[str]
    level_column: typing.Optional[bool]
    lines: typing.Optional[int]
    max_val: typing.Optional[float]
    # MaxDataPoints is the maximum number of data points that should be returned from a time series query.
    # NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
    # from the number of pixels visible in a visualization
    max_data_points: typing.Optional[int]
    min_val: typing.Optional[float]
    nodes: typing.Optional['NodesQuery']
    noise: typing.Optional[float]
    points: typing.Optional[list[list[object]]]
    pulse_wave: typing.Optional['PulseWaveQuery']
    # QueryType is an optional identifier for the type of query.
    # It can be used to distinguish different types of queries.
    query_type: typing.Optional[str]
    raw_frame_content: typing.Optional[str]
    # RefID is the unique identifier of the query, set by the frontend call.
    ref_id: typing.Optional[str]
    # Optionally define expected query result behavior
    result_assertions: typing.Optional['ResultAssertions']
    # Possible enum values:
    #  - `"annotations"` 
    #  - `"arrow"` 
    #  - `"csv_content"` 
    #  - `"csv_file"` 
    #  - `"csv_metric_values"` 
    #  - `"datapoints_outside_range"` 
    #  - `"exponential_heatmap_bucket_data"` 
    #  - `"flame_graph"` 
    #  - `"grafana_api"` 
    #  - `"linear_heatmap_bucket_data"` 
    #  - `"live"` 
    #  - `"logs"` 
    #  - `"manual_entry"` 
    #  - `"no_data_points"` 
    #  - `"node_graph"` 
    #  - `"predictable_csv_wave"` 
    #  - `"predictable_pulse"` 
    #  - `"random_walk"` 
    #  - `"random_walk_table"` 
    #  - `"random_walk_with_error"` 
    #  - `"raw_frame"` 
    #  - `"server_error_500"` 
    #  - `"simulation"` 
    #  - `"slow_query"` 
    #  - `"streaming_client"` 
    #  - `"table_static"` 
    #  - `"trace"` 
    #  - `"usa"` 
    #  - `"variables-query"` 
    scenario_id: typing.Optional[typing.Literal["annotations", "arrow", "csv_content", "csv_file", "csv_metric_values", "datapoints_outside_range", "exponential_heatmap_bucket_data", "flame_graph", "grafana_api", "linear_heatmap_bucket_data", "live", "logs", "manual_entry", "no_data_points", "node_graph", "predictable_csv_wave", "predictable_pulse", "random_walk", "random_walk_table", "random_walk_with_error", "raw_frame", "server_error_500", "simulation", "slow_query", "streaming_client", "table_static", "trace", "usa", "variables-query"]]
    series_count: typing.Optional[int]
    sim: typing.Optional['SimulationQuery']
    span_count: typing.Optional[int]
    spread: typing.Optional[float]
    start_value: typing.Optional[float]
    stream: typing.Optional['StreamingQuery']
    # common parameter used by many query types
    string_input: typing.Optional[str]
    # TimeRange represents the query range
    # NOTE: unlike generic /ds/query, we can now send explicit time values in each query
    # NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
    time_range: typing.Optional['TimeRange']
    usa: typing.Optional['USAQuery']
    with_nil: typing.Optional[bool]

    def __init__(self, alias: typing.Optional[str] = None, channel: typing.Optional[str] = None, csv_content: typing.Optional[str] = None, csv_file_name: typing.Optional[str] = None, csv_wave: typing.Optional[list['CSVWave']] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None, drop_percent: typing.Optional[float] = None, error_type: typing.Optional[typing.Literal["frontend_exception", "frontend_observable", "server_panic"]] = None, flamegraph_diff: typing.Optional[bool] = None, hide: typing.Optional[bool] = None, interval_ms: typing.Optional[float] = None, labels: typing.Optional[str] = None, level_column: typing.Optional[bool] = None, lines: typing.Optional[int] = None, max_val: typing.Optional[float] = None, max_data_points: typing.Optional[int] = None, min_val: typing.Optional[float] = None, nodes: typing.Optional['NodesQuery'] = None, noise: typing.Optional[float] = None, points: typing.Optional[list[list[object]]] = None, pulse_wave: typing.Optional['PulseWaveQuery'] = None, query_type: typing.Optional[str] = None, raw_frame_content: typing.Optional[str] = None, ref_id: typing.Optional[str] = None, result_assertions: typing.Optional['ResultAssertions'] = None, scenario_id: typing.Optional[typing.Literal["annotations", "arrow", "csv_content", "csv_file", "csv_metric_values", "datapoints_outside_range", "exponential_heatmap_bucket_data", "flame_graph", "grafana_api", "linear_heatmap_bucket_data", "live", "logs", "manual_entry", "no_data_points", "node_graph", "predictable_csv_wave", "predictable_pulse", "random_walk", "random_walk_table", "random_walk_with_error", "raw_frame", "server_error_500", "simulation", "slow_query", "streaming_client", "table_static", "trace", "usa", "variables-query"]] = None, series_count: typing.Optional[int] = None, sim: typing.Optional['SimulationQuery'] = None, span_count: typing.Optional[int] = None, spread: typing.Optional[float] = None, start_value: typing.Optional[float] = None, stream: typing.Optional['StreamingQuery'] = None, string_input: typing.Optional[str] = None, time_range: typing.Optional['TimeRange'] = None, usa: typing.Optional['USAQuery'] = None, with_nil: typing.Optional[bool] = None):
        self.alias = alias
        self.channel = channel
        self.csv_content = csv_content
        self.csv_file_name = csv_file_name
        self.csv_wave = csv_wave
        self.datasource = datasource
        self.drop_percent = drop_percent
        self.error_type = error_type
        self.flamegraph_diff = flamegraph_diff
        self.hide = hide
        self.interval_ms = interval_ms
        self.labels = labels
        self.level_column = level_column
        self.lines = lines
        self.max_val = max_val
        self.max_data_points = max_data_points
        self.min_val = min_val
        self.nodes = nodes
        self.noise = noise
        self.points = points
        self.pulse_wave = pulse_wave
        self.query_type = query_type
        self.raw_frame_content = raw_frame_content
        self.ref_id = ref_id
        self.result_assertions = result_assertions
        self.scenario_id = scenario_id
        self.series_count = series_count
        self.sim = sim
        self.span_count = span_count
        self.spread = spread
        self.start_value = start_value
        self.stream = stream
        self.string_input = string_input
        self.time_range = time_range
        self.usa = usa
        self.with_nil = with_nil

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.alias is not None:
            payload["alias"] = self.alias
        if self.channel is not None:
            payload["channel"] = self.channel
        if self.csv_content is not None:
            payload["csvContent"] = self.csv_content
        if self.csv_file_name is not None:
            payload["csvFileName"] = self.csv_file_name
        if self.csv_wave is not None:
            payload["csvWave"] = self.csv_wave
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.drop_percent is not None:
            payload["dropPercent"] = self.drop_percent
        if self.error_type is not None:
            payload["errorType"] = self.error_type
        if self.flamegraph_diff is not None:
            payload["flamegraphDiff"] = self.flamegraph_diff
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        if self.labels is not None:
            payload["labels"] = self.labels
        if self.level_column is not None:
            payload["levelColumn"] = self.level_column
        if self.lines is not None:
            payload["lines"] = self.lines
        if self.max_val is not None:
            payload["max"] = self.max_val
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.min_val is not None:
            payload["min"] = self.min_val
        if self.nodes is not None:
            payload["nodes"] = self.nodes
        if self.noise is not None:
            payload["noise"] = self.noise
        if self.points is not None:
            payload["points"] = self.points
        if self.pulse_wave is not None:
            payload["pulseWave"] = self.pulse_wave
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.raw_frame_content is not None:
            payload["rawFrameContent"] = self.raw_frame_content
        if self.ref_id is not None:
            payload["refId"] = self.ref_id
        if self.result_assertions is not None:
            payload["resultAssertions"] = self.result_assertions
        if self.scenario_id is not None:
            payload["scenarioId"] = self.scenario_id
        if self.series_count is not None:
            payload["seriesCount"] = self.series_count
        if self.sim is not None:
            payload["sim"] = self.sim
        if self.span_count is not None:
            payload["spanCount"] = self.span_count
        if self.spread is not None:
            payload["spread"] = self.spread
        if self.start_value is not None:
            payload["startValue"] = self.start_value
        if self.stream is not None:
            payload["stream"] = self.stream
        if self.string_input is not None:
            payload["stringInput"] = self.string_input
        if self.time_range is not None:
            payload["timeRange"] = self.time_range
        if self.usa is not None:
            payload["usa"] = self.usa
        if self.with_nil is not None:
            payload["withNil"] = self.with_nil
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "alias" in data:
            args["alias"] = data["alias"]
        if "channel" in data:
            args["channel"] = data["channel"]
        if "csvContent" in data:
            args["csv_content"] = data["csvContent"]
        if "csvFileName" in data:
            args["csv_file_name"] = data["csvFileName"]
        if "csvWave" in data:
            args["csv_wave"] = data["csvWave"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "dropPercent" in data:
            args["drop_percent"] = data["dropPercent"]
        if "errorType" in data:
            args["error_type"] = data["errorType"]
        if "flamegraphDiff" in data:
            args["flamegraph_diff"] = data["flamegraphDiff"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]
        if "labels" in data:
            args["labels"] = data["labels"]
        if "levelColumn" in data:
            args["level_column"] = data["levelColumn"]
        if "lines" in data:
            args["lines"] = data["lines"]
        if "max" in data:
            args["max_val"] = data["max"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "min" in data:
            args["min_val"] = data["min"]
        if "nodes" in data:
            args["nodes"] = NodesQuery.from_json(data["nodes"])
        if "noise" in data:
            args["noise"] = data["noise"]
        if "points" in data:
            args["points"] = data["points"]
        if "pulseWave" in data:
            args["pulse_wave"] = PulseWaveQuery.from_json(data["pulseWave"])
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "rawFrameContent" in data:
            args["raw_frame_content"] = data["rawFrameContent"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "resultAssertions" in data:
            args["result_assertions"] = ResultAssertions.from_json(data["resultAssertions"])
        if "scenarioId" in data:
            args["scenario_id"] = data["scenarioId"]
        if "seriesCount" in data:
            args["series_count"] = data["seriesCount"]
        if "sim" in data:
            args["sim"] = SimulationQuery.from_json(data["sim"])
        if "spanCount" in data:
            args["span_count"] = data["spanCount"]
        if "spread" in data:
            args["spread"] = data["spread"]
        if "startValue" in data:
            args["start_value"] = data["startValue"]
        if "stream" in data:
            args["stream"] = StreamingQuery.from_json(data["stream"])
        if "stringInput" in data:
            args["string_input"] = data["stringInput"]
        if "timeRange" in data:
            args["time_range"] = TimeRange.from_json(data["timeRange"])
        if "usa" in data:
            args["usa"] = USAQuery.from_json(data["usa"])
        if "withNil" in data:
            args["with_nil"] = data["withNil"]        

        return cls(**args)


def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="",
        from_json_hook=Dataquery.from_json,
    )






# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..models import common
import typing
from ..cog import runtime as cogruntime


class Options:
    show_labels: bool
    show_common_labels: bool
    show_time: bool
    show_log_context_toggle: bool
    wrap_log_message: bool
    prettify_log_message: bool
    enable_log_details: bool
    sort_order: common.LogsSortOrder
    dedup_strategy: common.LogsDedupStrategy
    # TODO: figure out how to define callbacks
    on_click_filter_label: typing.Optional[object]
    on_click_filter_out_label: typing.Optional[object]
    is_filter_label_active: typing.Optional[object]
    on_click_filter_string: typing.Optional[object]
    on_click_filter_out_string: typing.Optional[object]
    on_click_show_field: typing.Optional[object]
    on_click_hide_field: typing.Optional[object]
    displayed_fields: typing.Optional[list[str]]

    def __init__(self, show_labels: bool = False, show_common_labels: bool = False, show_time: bool = False, show_log_context_toggle: bool = False, wrap_log_message: bool = False, prettify_log_message: bool = False, enable_log_details: bool = False, sort_order: typing.Optional[common.LogsSortOrder] = None, dedup_strategy: typing.Optional[common.LogsDedupStrategy] = None, on_click_filter_label: typing.Optional[object] = None, on_click_filter_out_label: typing.Optional[object] = None, is_filter_label_active: typing.Optional[object] = None, on_click_filter_string: typing.Optional[object] = None, on_click_filter_out_string: typing.Optional[object] = None, on_click_show_field: typing.Optional[object] = None, on_click_hide_field: typing.Optional[object] = None, displayed_fields: typing.Optional[list[str]] = None):
        self.show_labels = show_labels
        self.show_common_labels = show_common_labels
        self.show_time = show_time
        self.show_log_context_toggle = show_log_context_toggle
        self.wrap_log_message = wrap_log_message
        self.prettify_log_message = prettify_log_message
        self.enable_log_details = enable_log_details
        self.sort_order = sort_order if sort_order is not None else common.LogsSortOrder.DESCENDING
        self.dedup_strategy = dedup_strategy if dedup_strategy is not None else common.LogsDedupStrategy.NONE
        self.on_click_filter_label = on_click_filter_label
        self.on_click_filter_out_label = on_click_filter_out_label
        self.is_filter_label_active = is_filter_label_active
        self.on_click_filter_string = on_click_filter_string
        self.on_click_filter_out_string = on_click_filter_out_string
        self.on_click_show_field = on_click_show_field
        self.on_click_hide_field = on_click_hide_field
        self.displayed_fields = displayed_fields

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "showLabels": self.show_labels,
            "showCommonLabels": self.show_common_labels,
            "showTime": self.show_time,
            "showLogContextToggle": self.show_log_context_toggle,
            "wrapLogMessage": self.wrap_log_message,
            "prettifyLogMessage": self.prettify_log_message,
            "enableLogDetails": self.enable_log_details,
            "sortOrder": self.sort_order,
            "dedupStrategy": self.dedup_strategy,
        }
        if self.on_click_filter_label is not None:
            payload["onClickFilterLabel"] = self.on_click_filter_label
        if self.on_click_filter_out_label is not None:
            payload["onClickFilterOutLabel"] = self.on_click_filter_out_label
        if self.is_filter_label_active is not None:
            payload["isFilterLabelActive"] = self.is_filter_label_active
        if self.on_click_filter_string is not None:
            payload["onClickFilterString"] = self.on_click_filter_string
        if self.on_click_filter_out_string is not None:
            payload["onClickFilterOutString"] = self.on_click_filter_out_string
        if self.on_click_show_field is not None:
            payload["onClickShowField"] = self.on_click_show_field
        if self.on_click_hide_field is not None:
            payload["onClickHideField"] = self.on_click_hide_field
        if self.displayed_fields is not None:
            payload["displayedFields"] = self.displayed_fields
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "showLabels" in data:
            args["show_labels"] = data["showLabels"]
        if "showCommonLabels" in data:
            args["show_common_labels"] = data["showCommonLabels"]
        if "showTime" in data:
            args["show_time"] = data["showTime"]
        if "showLogContextToggle" in data:
            args["show_log_context_toggle"] = data["showLogContextToggle"]
        if "wrapLogMessage" in data:
            args["wrap_log_message"] = data["wrapLogMessage"]
        if "prettifyLogMessage" in data:
            args["prettify_log_message"] = data["prettifyLogMessage"]
        if "enableLogDetails" in data:
            args["enable_log_details"] = data["enableLogDetails"]
        if "sortOrder" in data:
            args["sort_order"] = data["sortOrder"]
        if "dedupStrategy" in data:
            args["dedup_strategy"] = data["dedupStrategy"]
        if "onClickFilterLabel" in data:
            args["on_click_filter_label"] = data["onClickFilterLabel"]
        if "onClickFilterOutLabel" in data:
            args["on_click_filter_out_label"] = data["onClickFilterOutLabel"]
        if "isFilterLabelActive" in data:
            args["is_filter_label_active"] = data["isFilterLabelActive"]
        if "onClickFilterString" in data:
            args["on_click_filter_string"] = data["onClickFilterString"]
        if "onClickFilterOutString" in data:
            args["on_click_filter_out_string"] = data["onClickFilterOutString"]
        if "onClickShowField" in data:
            args["on_click_show_field"] = data["onClickShowField"]
        if "onClickHideField" in data:
            args["on_click_hide_field"] = data["onClickHideField"]
        if "displayedFields" in data:
            args["displayed_fields"] = data["displayedFields"]        

        return cls(**args)


def variant_config():
    return cogruntime.PanelCfgConfig(
        identifier="logs",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import alerting
from ..cog import variants as cogvariants


class Query(cogbuilder.Builder[alerting.Query]):    
    _internal: alerting.Query

    def __init__(self, ref_id: str):
        self._internal = alerting.Query()        
        self._internal.ref_id = ref_id

    def build(self) -> alerting.Query:
        return self._internal    
    
    def datasource_uid(self, datasource_uid: str) -> typing.Self:        
        self._internal.datasource_uid = datasource_uid
    
        return self
    
    def model(self, model: cogbuilder.Builder[cogvariants.Dataquery]) -> typing.Self:        
        model_resource = model.build()
        self._internal.model = model_resource
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:        
        self._internal.query_type = query_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:        
        self._internal.ref_id = ref_id
    
        return self
    
    def relative_time_range(self, relative_time_range: alerting.RelativeTimeRange) -> typing.Self:        
        self._internal.relative_time_range = relative_time_range
    
        return self
    

class RuleGroup(cogbuilder.Builder[alerting.RuleGroup]):    
    _internal: alerting.RuleGroup

    def __init__(self, title: str):
        self._internal = alerting.RuleGroup()        
        self._internal.title = title

    def build(self) -> alerting.RuleGroup:
        return self._internal    
    
    def folder_uid(self, folder_uid: str) -> typing.Self:        
        self._internal.folder_uid = folder_uid
    
        return self
    
    def interval(self, interval: alerting.Duration) -> typing.Self:    
        """
        The interval, in seconds, at which all rules in the group are evaluated.
        If a group contains many rules, the rules are evaluated sequentially.
        """
            
        self._internal.interval = interval
    
        return self
    
    def rules(self, rules: list[cogbuilder.Builder[alerting.Rule]]) -> typing.Self:        
        rules_resources = [r1.build() for r1 in rules]
        self._internal.rules = rules_resources
    
        return self
    
    def with_rule(self, rules: cogbuilder.Builder[alerting.Rule]) -> typing.Self:        
        if self._internal.rules is None:
            self._internal.rules = []
        
        rules_resource = rules.build()
        self._internal.rules.append(rules_resource)
    
        return self
    
    def title(self, title: str) -> typing.Self:        
        self._internal.title = title
    
        return self
    

class NotificationSettings(cogbuilder.Builder[alerting.NotificationSettings]):    
    _internal: alerting.NotificationSettings

    def __init__(self):
        self._internal = alerting.NotificationSettings()

    def build(self) -> alerting.NotificationSettings:
        return self._internal    
    
    def group_by(self, group_by: list[str]) -> typing.Self:        
        self._internal.group_by = group_by
    
        return self
    
    def group_interval(self, group_interval: str) -> typing.Self:        
        self._internal.group_interval = group_interval
    
        return self
    
    def group_wait(self, group_wait: str) -> typing.Self:        
        self._internal.group_wait = group_wait
    
        return self
    
    def mute_time_intervals(self, mute_time_intervals: list[str]) -> typing.Self:        
        self._internal.mute_time_intervals = mute_time_intervals
    
        return self
    
    def receiver(self, receiver: str) -> typing.Self:        
        self._internal.receiver = receiver
    
        return self
    
    def repeat_interval(self, repeat_interval: str) -> typing.Self:        
        self._internal.repeat_interval = repeat_interval
    
        return self
    

class ContactPoint(cogbuilder.Builder[alerting.ContactPoint]):    
    """
    EmbeddedContactPoint is the contact point type that is used
    by grafanas embedded alertmanager implementation.
    """
    
    _internal: alerting.ContactPoint

    def __init__(self):
        self._internal = alerting.ContactPoint()

    def build(self) -> alerting.ContactPoint:
        return self._internal    
    
    def disable_resolve_message(self, disable_resolve_message: bool) -> typing.Self:    
        """
        EmbeddedContactPoint is the contact point type that is used
        by grafanas embedded alertmanager implementation.
        """
            
        self._internal.disable_resolve_message = disable_resolve_message
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        """
        EmbeddedContactPoint is the contact point type that is used
        by grafanas embedded alertmanager implementation.
        """
            
        self._internal.name = name
    
        return self
    
    def provenance(self, provenance: str) -> typing.Self:    
        """
        EmbeddedContactPoint is the contact point type that is used
        by grafanas embedded alertmanager implementation.
        """
            
        self._internal.provenance = provenance
    
        return self
    
    def settings(self, settings: alerting.Json) -> typing.Self:    
        """
        EmbeddedContactPoint is the contact point type that is used
        by grafanas embedded alertmanager implementation.
        """
            
        self._internal.settings = settings
    
        return self
    
    def type_val(self, type_val: typing.Literal["alertmanager", " dingding", " discord", " email", " googlechat", " kafka", " line", " opsgenie", " pagerduty", " pushover", " sensugo", " slack", " teams", " telegram", " threema", " victorops", " webhook", " wecom"]) -> typing.Self:    
        """
        EmbeddedContactPoint is the contact point type that is used
        by grafanas embedded alertmanager implementation.
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def uid(self, uid: str) -> typing.Self:    
        """
        EmbeddedContactPoint is the contact point type that is used
        by grafanas embedded alertmanager implementation.
        """
            
        if not len(uid) >= 1:
            raise ValueError("len(uid) must be >= 1")
        if not len(uid) <= 40:
            raise ValueError("len(uid) must be <= 40")
        self._internal.uid = uid
    
        return self
    

class Matcher(cogbuilder.Builder[alerting.Matcher]):    
    _internal: alerting.Matcher

    def __init__(self):
        self._internal = alerting.Matcher()

    def build(self) -> alerting.Matcher:
        return self._internal    
    
    def name(self, name: str) -> typing.Self:        
        self._internal.name = name
    
        return self
    
    def type(self, type: alerting.MatchType) -> typing.Self:        
        self._internal.type = type
    
        return self
    
    def value(self, value: str) -> typing.Self:        
        self._internal.value = value
    
        return self
    

class MuteTiming(cogbuilder.Builder[alerting.MuteTiming]):    
    _internal: alerting.MuteTiming

    def __init__(self):
        self._internal = alerting.MuteTiming()

    def build(self) -> alerting.MuteTiming:
        return self._internal    
    
    def name(self, name: str) -> typing.Self:        
        self._internal.name = name
    
        return self
    
    def time_intervals(self, time_intervals: list[cogbuilder.Builder[alerting.TimeInterval]]) -> typing.Self:        
        time_intervals_resources = [r1.build() for r1 in time_intervals]
        self._internal.time_intervals = time_intervals_resources
    
        return self
    

class NotificationTemplate(cogbuilder.Builder[alerting.NotificationTemplate]):    
    _internal: alerting.NotificationTemplate

    def __init__(self):
        self._internal = alerting.NotificationTemplate()

    def build(self) -> alerting.NotificationTemplate:
        return self._internal    
    
    def name(self, name: str) -> typing.Self:        
        self._internal.name = name
    
        return self
    
    def provenance(self, provenance: alerting.Provenance) -> typing.Self:        
        self._internal.provenance = provenance
    
        return self
    
    def template(self, template: str) -> typing.Self:        
        self._internal.template = template
    
        return self
    
    def version(self, version: str) -> typing.Self:        
        self._internal.version = version
    
        return self
    

class Rule(cogbuilder.Builder[alerting.Rule]):    
    _internal: alerting.Rule

    def __init__(self, title: str):
        self._internal = alerting.Rule()        
        if not len(title) >= 1:
            raise ValueError("len(title) must be >= 1")
        if not len(title) <= 190:
            raise ValueError("len(title) must be <= 190")
        self._internal.title = title

    def build(self) -> alerting.Rule:
        return self._internal    
    
    def annotations(self, annotations: dict[str, str]) -> typing.Self:        
        self._internal.annotations = annotations
    
        return self
    
    def condition(self, condition: str) -> typing.Self:        
        self._internal.condition = condition
    
        return self
    
    def queries(self, data: list[cogbuilder.Builder[alerting.Query]]) -> typing.Self:        
        data_resources = [r1.build() for r1 in data]
        self._internal.data = data_resources
    
        return self
    
    def with_query(self, data: cogbuilder.Builder[alerting.Query]) -> typing.Self:        
        if self._internal.data is None:
            self._internal.data = []
        
        data_resource = data.build()
        self._internal.data.append(data_resource)
    
        return self
    
    def exec_err_state(self, exec_err_state: typing.Literal["OK", "Alerting", "Error"]) -> typing.Self:        
        self._internal.exec_err_state = exec_err_state
    
        return self
    
    def folder_uid(self, folder_uid: str) -> typing.Self:        
        self._internal.folder_uid = folder_uid
    
        return self
    
    def for_val(self, for_val: str) -> typing.Self:    
        """
        The amount of time, in seconds, for which the rule must be breached for the rule to be considered to be Firing.
        Before this time has elapsed, the rule is only considered to be Pending.
        """
            
        self._internal.for_val = for_val
    
        return self
    
    def id_val(self, id_val: int) -> typing.Self:        
        self._internal.id_val = id_val
    
        return self
    
    def is_paused(self, is_paused: bool) -> typing.Self:        
        self._internal.is_paused = is_paused
    
        return self
    
    def labels(self, labels: dict[str, str]) -> typing.Self:        
        self._internal.labels = labels
    
        return self
    
    def no_data_state(self, no_data_state: typing.Literal["Alerting", "NoData", "OK"]) -> typing.Self:        
        self._internal.no_data_state = no_data_state
    
        return self
    
    def notification_settings(self, notification_settings: cogbuilder.Builder[alerting.NotificationSettings]) -> typing.Self:        
        notification_settings_resource = notification_settings.build()
        self._internal.notification_settings = notification_settings_resource
    
        return self
    
    def org_id(self, org_id: int) -> typing.Self:        
        self._internal.org_id = org_id
    
        return self
    
    def provenance(self, provenance: alerting.Provenance) -> typing.Self:        
        self._internal.provenance = provenance
    
        return self
    
    def record(self, record: cogbuilder.Builder[alerting.RecordRule]) -> typing.Self:        
        record_resource = record.build()
        self._internal.record = record_resource
    
        return self
    
    def rule_group(self, rule_group: str) -> typing.Self:        
        if not len(rule_group) >= 1:
            raise ValueError("len(rule_group) must be >= 1")
        if not len(rule_group) <= 190:
            raise ValueError("len(rule_group) must be <= 190")
        self._internal.rule_group = rule_group
    
        return self
    
    def title(self, title: str) -> typing.Self:        
        if not len(title) >= 1:
            raise ValueError("len(title) must be >= 1")
        if not len(title) <= 190:
            raise ValueError("len(title) must be <= 190")
        self._internal.title = title
    
        return self
    
    def uid(self, uid: str) -> typing.Self:        
        if not len(uid) >= 1:
            raise ValueError("len(uid) must be >= 1")
        if not len(uid) <= 40:
            raise ValueError("len(uid) must be <= 40")
        self._internal.uid = uid
    
        return self
    
    def updated(self, updated: str) -> typing.Self:        
        self._internal.updated = updated
    
        return self
    

class RecordRule(cogbuilder.Builder[alerting.RecordRule]):    
    _internal: alerting.RecordRule

    def __init__(self):
        self._internal = alerting.RecordRule()

    def build(self) -> alerting.RecordRule:
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:        
        self._internal.from_val = from_val
    
        return self
    
    def metric(self, metric: str) -> typing.Self:        
        self._internal.metric = metric
    
        return self
    

class NotificationPolicy(cogbuilder.Builder[alerting.NotificationPolicy]):    
    """
    A Route is a node that contains definitions of how to handle alerts. This is modified
    from the upstream alertmanager in that it adds the ObjectMatchers property.
    """
    
    _internal: alerting.NotificationPolicy

    def __init__(self):
        self._internal = alerting.NotificationPolicy()

    def build(self) -> alerting.NotificationPolicy:
        return self._internal    
    
    def continue_val(self, continue_val: bool) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.continue_val = continue_val
    
        return self
    
    def group_by(self, group_by: list[str]) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.group_by = group_by
    
        return self
    
    def group_interval(self, group_interval: str) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.group_interval = group_interval
    
        return self
    
    def group_wait(self, group_wait: str) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.group_wait = group_wait
    
        return self
    
    def match(self, match: dict[str, str]) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.match = match
    
        return self
    
    def match_re(self, match_re: alerting.MatchRegexps) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.match_re = match_re
    
        return self
    
    def matchers(self, matchers: alerting.Matchers) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.matchers = matchers
    
        return self
    
    def mute_time_intervals(self, mute_time_intervals: list[str]) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.mute_time_intervals = mute_time_intervals
    
        return self
    
    def object_matchers(self, object_matchers: alerting.ObjectMatchers) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.object_matchers = object_matchers
    
        return self
    
    def provenance(self, provenance: alerting.Provenance) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.provenance = provenance
    
        return self
    
    def receiver(self, receiver: str) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.receiver = receiver
    
        return self
    
    def repeat_interval(self, repeat_interval: str) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        self._internal.repeat_interval = repeat_interval
    
        return self
    
    def routes(self, routes: list[cogbuilder.Builder[alerting.NotificationPolicy]]) -> typing.Self:    
        """
        A Route is a node that contains definitions of how to handle alerts. This is modified
        from the upstream alertmanager in that it adds the ObjectMatchers property.
        """
            
        routes_resources = [r1.build() for r1 in routes]
        self._internal.routes = routes_resources
    
        return self
    

class TimeInterval(cogbuilder.Builder[alerting.TimeInterval]):    
    _internal: alerting.TimeInterval

    def __init__(self):
        self._internal = alerting.TimeInterval()

    def build(self) -> alerting.TimeInterval:
        return self._internal    
    
    def name(self, name: str) -> typing.Self:        
        self._internal.name = name
    
        return self
    
    def time_intervals(self, time_intervals: list[cogbuilder.Builder[alerting.TimeIntervalItem]]) -> typing.Self:        
        time_intervals_resources = [r1.build() for r1 in time_intervals]
        self._internal.time_intervals = time_intervals_resources
    
        return self
    

class TimeIntervalItem(cogbuilder.Builder[alerting.TimeIntervalItem]):    
    _internal: alerting.TimeIntervalItem

    def __init__(self):
        self._internal = alerting.TimeIntervalItem()

    def build(self) -> alerting.TimeIntervalItem:
        return self._internal    
    
    def days_of_month(self, days_of_month: list[str]) -> typing.Self:        
        self._internal.days_of_month = days_of_month
    
        return self
    
    def location(self, location: str) -> typing.Self:        
        self._internal.location = location
    
        return self
    
    def months(self, months: list[str]) -> typing.Self:        
        self._internal.months = months
    
        return self
    
    def times(self, times: list[cogbuilder.Builder[alerting.TimeIntervalTimeRange]]) -> typing.Self:        
        times_resources = [r1.build() for r1 in times]
        self._internal.times = times_resources
    
        return self
    
    def weekdays(self, weekdays: list[str]) -> typing.Self:        
        self._internal.weekdays = weekdays
    
        return self
    
    def years(self, years: list[str]) -> typing.Self:        
        self._internal.years = years
    
        return self
    

class TimeIntervalTimeRange(cogbuilder.Builder[alerting.TimeIntervalTimeRange]):    
    _internal: alerting.TimeIntervalTimeRange

    def __init__(self):
        self._internal = alerting.TimeIntervalTimeRange()

    def build(self) -> alerting.TimeIntervalTimeRange:
        return self._internal    
    
    def end_time(self, end_time: str) -> typing.Self:        
        self._internal.end_time = end_time
    
        return self
    
    def start_time(self, start_time: str) -> typing.Self:        
        self._internal.start_time = start_time
    
        return self
    


# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import testdata
from ..models import dashboard


class CSVWave(cogbuilder.Builder[testdata.CSVWave]):    
    _internal: testdata.CSVWave

    def __init__(self):
        self._internal = testdata.CSVWave()

    def build(self) -> testdata.CSVWave:
        return self._internal    
    
    def labels(self, labels: str) -> typing.Self:        
        self._internal.labels = labels
    
        return self
    
    def name(self, name: str) -> typing.Self:        
        self._internal.name = name
    
        return self
    
    def time_step(self, time_step: int) -> typing.Self:        
        self._internal.time_step = time_step
    
        return self
    
    def values_csv(self, values_csv: str) -> typing.Self:        
        self._internal.values_csv = values_csv
    
        return self
    

class NodesQuery(cogbuilder.Builder[testdata.NodesQuery]):    
    _internal: testdata.NodesQuery

    def __init__(self):
        self._internal = testdata.NodesQuery()

    def build(self) -> testdata.NodesQuery:
        return self._internal    
    
    def count(self, count: int) -> typing.Self:        
        self._internal.count = count
    
        return self
    
    def seed(self, seed: int) -> typing.Self:        
        self._internal.seed = seed
    
        return self
    
    def type_val(self, type_val: typing.Literal["random", "random edges", "response_medium", "response_small", "feature_showcase"]) -> typing.Self:    
        """
        Possible enum values:
         - `"random"` 
         - `"random edges"` 
         - `"response_medium"` 
         - `"response_small"` 
         - `"feature_showcase"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    

class PulseWaveQuery(cogbuilder.Builder[testdata.PulseWaveQuery]):    
    _internal: testdata.PulseWaveQuery

    def __init__(self):
        self._internal = testdata.PulseWaveQuery()

    def build(self) -> testdata.PulseWaveQuery:
        return self._internal    
    
    def off_count(self, off_count: int) -> typing.Self:        
        self._internal.off_count = off_count
    
        return self
    
    def off_value(self, off_value: float) -> typing.Self:        
        self._internal.off_value = off_value
    
        return self
    
    def on_count(self, on_count: int) -> typing.Self:        
        self._internal.on_count = on_count
    
        return self
    
    def on_value(self, on_value: float) -> typing.Self:        
        self._internal.on_value = on_value
    
        return self
    
    def time_step(self, time_step: int) -> typing.Self:        
        self._internal.time_step = time_step
    
        return self
    

class ResultAssertions(cogbuilder.Builder[testdata.ResultAssertions]):    
    _internal: testdata.ResultAssertions

    def __init__(self):
        self._internal = testdata.ResultAssertions()

    def build(self) -> testdata.ResultAssertions:
        return self._internal    
    
    def max_frames(self, max_frames: int) -> typing.Self:    
        """
        Maximum frame count
        """
            
        self._internal.max_frames = max_frames
    
        return self
    
    def type_val(self, type_val: typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]) -> typing.Self:    
        """
        Type asserts that the frame matches a known type structure.
        Possible enum values:
         - `""` 
         - `"timeseries-wide"` 
         - `"timeseries-long"` 
         - `"timeseries-many"` 
         - `"timeseries-multi"` 
         - `"directory-listing"` 
         - `"table"` 
         - `"numeric-wide"` 
         - `"numeric-multi"` 
         - `"numeric-long"` 
         - `"log-lines"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def type_version(self, type_version: list[int]) -> typing.Self:    
        """
        TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
        contract documentation https://grafana.github.io/dataplane/contract/.
        """
            
        self._internal.type_version = type_version
    
        return self
    

class Key(cogbuilder.Builder[testdata.Key]):    
    _internal: testdata.Key

    def __init__(self):
        self._internal = testdata.Key()

    def build(self) -> testdata.Key:
        return self._internal    
    
    def tick(self, tick: float) -> typing.Self:        
        self._internal.tick = tick
    
        return self
    
    def type_val(self, type_val: str) -> typing.Self:        
        self._internal.type_val = type_val
    
        return self
    
    def uid(self, uid: str) -> typing.Self:        
        self._internal.uid = uid
    
        return self
    

class SimulationQuery(cogbuilder.Builder[testdata.SimulationQuery]):    
    _internal: testdata.SimulationQuery

    def __init__(self):
        self._internal = testdata.SimulationQuery()

    def build(self) -> testdata.SimulationQuery:
        return self._internal    
    
    def config(self, config: object) -> typing.Self:        
        self._internal.config = config
    
        return self
    
    def key(self, key: cogbuilder.Builder[testdata.Key]) -> typing.Self:        
        key_resource = key.build()
        self._internal.key = key_resource
    
        return self
    
    def last(self, last: bool) -> typing.Self:        
        self._internal.last = last
    
        return self
    
    def stream(self, stream: bool) -> typing.Self:        
        self._internal.stream = stream
    
        return self
    

class StreamingQuery(cogbuilder.Builder[testdata.StreamingQuery]):    
    _internal: testdata.StreamingQuery

    def __init__(self):
        self._internal = testdata.StreamingQuery()

    def build(self) -> testdata.StreamingQuery:
        return self._internal    
    
    def bands(self, bands: int) -> typing.Self:        
        self._internal.bands = bands
    
        return self
    
    def noise(self, noise: float) -> typing.Self:        
        self._internal.noise = noise
    
        return self
    
    def speed(self, speed: float) -> typing.Self:        
        self._internal.speed = speed
    
        return self
    
    def spread(self, spread: float) -> typing.Self:        
        self._internal.spread = spread
    
        return self
    
    def type_val(self, type_val: typing.Literal["fetch", "logs", "signal", "traces"]) -> typing.Self:    
        """
        Possible enum values:
         - `"fetch"` 
         - `"logs"` 
         - `"signal"` 
         - `"traces"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def url(self, url: str) -> typing.Self:        
        self._internal.url = url
    
        return self
    

class TimeRange(cogbuilder.Builder[testdata.TimeRange]):    
    _internal: testdata.TimeRange

    def __init__(self):
        self._internal = testdata.TimeRange()

    def build(self) -> testdata.TimeRange:
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        """
        From is the start time of the query.
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        """
        To is the end time of the query.
        """
            
        self._internal.to = to
    
        return self
    

class USAQuery(cogbuilder.Builder[testdata.USAQuery]):    
    _internal: testdata.USAQuery

    def __init__(self):
        self._internal = testdata.USAQuery()

    def build(self) -> testdata.USAQuery:
        return self._internal    
    
    def fields(self, fields: list[str]) -> typing.Self:        
        self._internal.fields = fields
    
        return self
    
    def mode(self, mode: str) -> typing.Self:        
        self._internal.mode = mode
    
        return self
    
    def period(self, period: str) -> typing.Self:        
        self._internal.period = period
    
        return self
    
    def states(self, states: list[str]) -> typing.Self:        
        self._internal.states = states
    
        return self
    

class Dataquery(cogbuilder.Builder[testdata.Dataquery]):    
    _internal: testdata.Dataquery

    def __init__(self):
        self._internal = testdata.Dataquery()

    def build(self) -> testdata.Dataquery:
        return self._internal    
    
    def alias(self, alias: str) -> typing.Self:        
        self._internal.alias = alias
    
        return self
    
    def channel(self, channel: str) -> typing.Self:    
        """
        Used for live query
        """
            
        self._internal.channel = channel
    
        return self
    
    def csv_content(self, csv_content: str) -> typing.Self:        
        self._internal.csv_content = csv_content
    
        return self
    
    def csv_file_name(self, csv_file_name: str) -> typing.Self:        
        self._internal.csv_file_name = csv_file_name
    
        return self
    
    def csv_wave(self, csv_wave: list[cogbuilder.Builder[testdata.CSVWave]]) -> typing.Self:        
        csv_wave_resources = [r1.build() for r1 in csv_wave]
        self._internal.csv_wave = csv_wave_resources
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def drop_percent(self, drop_percent: float) -> typing.Self:    
        """
        Drop percentage (the chance we will lose a point 0-100)
        """
            
        self._internal.drop_percent = drop_percent
    
        return self
    
    def error_type(self, error_type: typing.Literal["frontend_exception", "frontend_observable", "server_panic"]) -> typing.Self:    
        """
        Possible enum values:
         - `"frontend_exception"` 
         - `"frontend_observable"` 
         - `"server_panic"` 
        """
            
        self._internal.error_type = error_type
    
        return self
    
    def flamegraph_diff(self, flamegraph_diff: bool) -> typing.Self:        
        self._internal.flamegraph_diff = flamegraph_diff
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        NOTE: this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Interval is the suggested duration between time points in a time series query.
        NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
        from the interval required to fill a pixels in the visualization
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    
    def labels(self, labels: str) -> typing.Self:        
        self._internal.labels = labels
    
        return self
    
    def level_column(self, level_column: bool) -> typing.Self:        
        self._internal.level_column = level_column
    
        return self
    
    def lines(self, lines: int) -> typing.Self:        
        self._internal.lines = lines
    
        return self
    
    def max_val(self, max_val: float) -> typing.Self:        
        self._internal.max_val = max_val
    
        return self
    
    def max_data_points(self, max_data_points: int) -> typing.Self:    
        """
        MaxDataPoints is the maximum number of data points that should be returned from a time series query.
        NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
        from the number of pixels visible in a visualization
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def min_val(self, min_val: float) -> typing.Self:        
        self._internal.min_val = min_val
    
        return self
    
    def nodes(self, nodes: cogbuilder.Builder[testdata.NodesQuery]) -> typing.Self:        
        nodes_resource = nodes.build()
        self._internal.nodes = nodes_resource
    
        return self
    
    def noise(self, noise: float) -> typing.Self:        
        self._internal.noise = noise
    
        return self
    
    def points(self, points: list[list[object]]) -> typing.Self:        
        self._internal.points = points
    
        return self
    
    def pulse_wave(self, pulse_wave: cogbuilder.Builder[testdata.PulseWaveQuery]) -> typing.Self:        
        pulse_wave_resource = pulse_wave.build()
        self._internal.pulse_wave = pulse_wave_resource
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def raw_frame_content(self, raw_frame_content: str) -> typing.Self:        
        self._internal.raw_frame_content = raw_frame_content
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def result_assertions(self, result_assertions: cogbuilder.Builder[testdata.ResultAssertions]) -> typing.Self:    
        """
        Optionally define expected query result behavior
        """
            
        result_assertions_resource = result_assertions.build()
        self._internal.result_assertions = result_assertions_resource
    
        return self
    
    def scenario_id(self, scenario_id: typing.Literal["annotations", "arrow", "csv_content", "csv_file", "csv_metric_values", "datapoints_outside_range", "exponential_heatmap_bucket_data", "flame_graph", "grafana_api", "linear_heatmap_bucket_data", "live", "logs", "manual_entry", "no_data_points", "node_graph", "predictable_csv_wave", "predictable_pulse", "random_walk", "random_walk_table", "random_walk_with_error", "raw_frame", "server_error_500", "simulation", "slow_query", "streaming_client", "table_static", "trace", "usa", "variables-query"]) -> typing.Self:    
        """
        Possible enum values:
         - `"annotations"` 
         - `"arrow"` 
         - `"csv_content"` 
         - `"csv_file"` 
         - `"csv_metric_values"` 
         - `"datapoints_outside_range"` 
         - `"exponential_heatmap_bucket_data"` 
         - `"flame_graph"` 
         - `"grafana_api"` 
         - `"linear_heatmap_bucket_data"` 
         - `"live"` 
         - `"logs"` 
         - `"manual_entry"` 
         - `"no_data_points"` 
         - `"node_graph"` 
         - `"predictable_csv_wave"` 
         - `"predictable_pulse"` 
         - `"random_walk"` 
         - `"random_walk_table"` 
         - `"random_walk_with_error"` 
         - `"raw_frame"` 
         - `"server_error_500"` 
         - `"simulation"` 
         - `"slow_query"` 
         - `"streaming_client"` 
         - `"table_static"` 
         - `"trace"` 
         - `"usa"` 
         - `"variables-query"` 
        """
            
        self._internal.scenario_id = scenario_id
    
        return self
    
    def series_count(self, series_count: int) -> typing.Self:        
        self._internal.series_count = series_count
    
        return self
    
    def sim(self, sim: cogbuilder.Builder[testdata.SimulationQuery]) -> typing.Self:        
        sim_resource = sim.build()
        self._internal.sim = sim_resource
    
        return self
    
    def span_count(self, span_count: int) -> typing.Self:        
        self._internal.span_count = span_count
    
        return self
    
    def spread(self, spread: float) -> typing.Self:        
        self._internal.spread = spread
    
        return self
    
    def start_value(self, start_value: float) -> typing.Self:        
        self._internal.start_value = start_value
    
        return self
    
    def stream(self, stream: cogbuilder.Builder[testdata.StreamingQuery]) -> typing.Self:        
        stream_resource = stream.build()
        self._internal.stream = stream_resource
    
        return self
    
    def string_input(self, string_input: str) -> typing.Self:    
        """
        common parameter used by many query types
        """
            
        self._internal.string_input = string_input
    
        return self
    
    def time_range(self, time_range: cogbuilder.Builder[testdata.TimeRange]) -> typing.Self:    
        """
        TimeRange represents the query range
        NOTE: unlike generic /ds/query, we can now send explicit time values in each query
        NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
        """
            
        time_range_resource = time_range.build()
        self._internal.time_range = time_range_resource
    
        return self
    
    def usa(self, usa: cogbuilder.Builder[testdata.USAQuery]) -> typing.Self:        
        usa_resource = usa.build()
        self._internal.usa = usa_resource
    
        return self
    
    def with_nil(self, with_nil: bool) -> typing.Self:        
        self._internal.with_nil = with_nil
    
        return self
    